pico-8 cartridge // http://www.pico-8.com
version 16
__lua__
-- home and hearth
-- squirrel eiserloh
cls()

g_show_stats = false -- cpu/mem
g_show_log_lines = 0 -- 0=none

-- set initial state to one of:
--  test_actors, test_pcg
--  test_mobs, test_quests
--  main_menu
g_state = "main_menu"

g_turn = 1
g_frame = 0


-------------------------------
function _init()
 
 rtime = time
 init_utils()
 init_actors()
 init_monsters()
 init_guild()
 init_menus()
	tick_quests()
  
end


-------------------------------
function _update()

 if( btn(⬅️) and btn(➡️) ) g_state="test_pcg"
 if( btn(⬅️) and btn(⬆️) ) g_state="test_mobs"
 if( btn(⬅️) and btn(⬇️) ) g_state="test_actors"
 if( btn(⬇️) and btn(⬆️) ) g_state="main_menu"
 if( btnp(⬇️) and btn(➡️) ) randomize_guildhall()

 g_frame += 1
 if( g_state=="test_actors" ) update_test_actors()
 if( g_state=="test_pcg" ) 			update_test_pcg()
 if( g_state=="test_mobs" ) 		update_test_mobs()
 if( g_state=="main_menu" ) 		update_main_menu()
 
end


-------------------------------
function _draw()

 if( g_state=="test_actors" ) draw_test_actors()
 if( g_state=="test_pcg" ) 			draw_test_pcg()
 if( g_state=="test_mobs" ) 		draw_test_mobs()
 if( g_state=="main_menu" ) 		draw_main_menu()

 if g_show_stats then
		draw_stats()
	end
	
	if g_show_log_lines > 0 then
	 draw_log()
	end

end


-->8
-- guild & guildhall

g_guild = {}
g_main_menus = {}

-- hall quad usage aliases
unused = 0
tables = 1
rooms  = 2


-------------------------------
function init_menus()

 init_main_menus()

end


-------------------------------
function init_main_menus()

 local members = {}
 members.name    = "members"
 members.tooltip = "manage guild ~amembership"
 members.update  = update_menu_members
 members.draw    = draw_menu_members
 members.current	= 1
 
 local quests = {}
 quests.name				= "quests"
 quests.tooltip = "hire a party for a ~aquest"
 quests.update  = update_menu_quests
 quests.draw    = draw_menu_quests
 quests.current	= 1
 
 local remodel = {}
 remodel.name    = "remodel"
 remodel.tooltip = "remodel or expand the ~aguildhall"
 remodel.update  = update_menu_remodel
 remodel.draw    = draw_menu_remodel
 remodel.current	= 1
 
 local bylaws = {}
 bylaws.name    = "bylaws"
 bylaws.tooltip = "modify guild ~abylaws"
 bylaws.update  = update_menu_bylaws
 bylaws.draw    = draw_menu_bylaws
 bylaws.current	= 1
 
 add( g_main_menus, members )
 add( g_main_menus, quests )
 add( g_main_menus, remodel )
 add( g_main_menus, bylaws )
 g_main_menus.members = members
 g_main_menus.quests 	= quests
 g_main_menus.remodel = remodel
 g_main_menus.bylaws 	= bylaws
 
 g_main_menus.current = 1
 g_main_menus.submenu = 0

end


-------------------------------
function init_guild()

 g_guild.quads = {}
 local quad1 = { idx=1, usage=tables }
 local quad2 = { idx=2, usage=unused }
 local quad3 = { idx=3, usage=unused }
 local quad4 = { idx=4, usage=unused }
 
 add( g_guild.quads, quad1 )
 add( g_guild.quads, quad2 )
 add( g_guild.quads, quad3 )
 add( g_guild.quads, quad4 )
 
 g_guild.money 		= 1000
 g_guild.payrate = 3
 g_guild.rent    = 10
 g_guild.seats 		= {}
 g_guild.rooms 		= {}
 g_guild.members = {}
 
 refresh_guild()

 -- starting members
 for i=1,8 do add_new_member() end

 refresh_guild()

end


-------------------------------
function refresh_guild()

 -- refresh_quads
 for quad in all( g_guild.quads ) do
	 refresh_quad( quad )
 end

 -- refresh seat xy positions
 g_guild.seats = {}
 g_guild.rooms = {}
 add_seats_for_quad( 1 )
 add_seats_for_quad( 2 )
 add_seats_for_quad( 3 )
 add_seats_for_quad( 4 )
 
 -- refresh seat assignments
 for member in all( g_guild.members ) do
  assign_seat( member )
 end
 
end


-------------------------------
function add_seats_for_quad( q )

 local quad = g_guild.quads[ q ]
 local minx,miny = 0,32
 
 if quad.usage == tables then
	 if( q==3 or q==4 ) minx += 48
 	if( q==2 or q==4 ) miny += 24
  add_seat( minx+16, miny+8 )
  add_seat( minx+24, miny+16 )
  add_seat( minx+16, miny+24 )
  add_seat( minx+8,  miny+16 )
  add_seat( minx+40, miny+8 )
  add_seat( minx+48, miny+16 )
  add_seat( minx+40, miny+24 )
  add_seat( minx+32, miny+16 )
 elseif quad.usage == rooms then
	 if( q==3 or q==4 ) minx += 56
 	if( q==2 or q==4 ) miny += 32
  add_room( minx+8,  miny+8 )
  add_room( minx+24, miny+8 )
  add_room( minx+40, miny+8 )
 end

end


-------------------------------
function add_seat( x,y )
 local seat = {}
 seat.x = x
 seat.y = y
 seat.occupant = nil
 add( g_guild.seats, seat )
end


-------------------------------
function add_room( x,y )
 local room = {}
 room.x = x
 room.y = y
 room.occupant = nil
 add( g_guild.rooms, room )
end


-------------------------------
function assign_seat( member )

	if member.tenant then
	 assign_room( member )
	 return
	end
	
	local seats = g_guild.seats
	for i=1,#seats do
	 local seat = seats[i]
	 if not seat.occupant then
	  seat.occupant = member
	  member.seatx = seat.x
	  member.seaty = seat.y
	  return
	 end
	end

end


-------------------------------
function assign_room( tenant )
	
	local rooms = g_guild.rooms
	for i=1,#rooms do
	 local room = rooms[i]
	 if not room.occupant then
	  room.occupant = member
	  return
	 end
	end

end


-------------------------------
function update_main_menu()

 local mm = g_main_menus

 -- submenu update instead if open
 if mm.submenu > 0 then
  if btnp(🅾️) then
	  sfx( sound_menu_back )
   mm.submenu = 0
  else
	  local sub_menu = mm[ mm.submenu ]
	  sub_menu.update( true )
  end
	  return 
 end
 
 -- ⬆️/⬇️ in main menu
 mm.current = up_down_wraparound( mm.current, #mm )
 local menu = mm[ mm.current ]

 -- ❎ in main menu -> submenu 
 if btnp(❎) then
  sfx( sound_menu_select )
  mm.submenu = g_main_menus.current
  local sub_menu = mm[ mm.submenu ]
  sub_menu.update( true )
  return
 end
 
 -- 🅾️ in main menu -> end turn
 if btnp(🅾️) then
	 end_turn()
	 return
	end
	
	menu.update( false )
	
end


-------------------------------
function draw_main_menu()

 cls(0) -- asdf
 
 draw_guildhall()
 draw_guild_stats()
 draw_info_bar()
 draw_menus()
  
end


------------------------------
function draw_guildhall()

 palt( 0, false )

 for quad_idx = 1,4 do
  local quad = g_guild.quads[ quad_idx ]
		map( quad.cx,quad.cy, quad.sx,quad.sy, 7,4 )
 end

--debug_draw_seats()
 
 -- draw members
 for actor in all( g_guild.members ) do
  draw_actor( actor )
 end

end


------------------------------
function draw_guild_stats()

 local g = g_guild
 local nm = #g.members
 local mm = #g.seats + #g.rooms
 local nt = get_num_tenants()
 local mt = #g.rooms
 rectfill( 0,0, 127,18, 1 )

 local money = add_commas( g.money )
 local pay = get_weekly_payroll()
 local rent = get_weekly_rent()
 print( "$"..money, 1,1, 9 )
 print( nm.."/"..mm.." members", 36,1, 11 )
 rprint( "week "..g_turn, 128,1, 3 )

 mprint( "pay ~8-$"..pay.."~6/wk", 1,7, 6 )
 mprint( "(~8$"..g.payrate.."~6 x "..nm.." members)", 128,7, 6, "right" )

 mprint( "rent ~b+$"..rent.."~6/wk", 1,13, 6 )
 mprint( "(~b$"..g.rent.."~6 x "..nt.." tenants)", 128,13, 6, "right" )

-- mprint( "empty?  or is this an info line?", 1,20, 1 )
 local menu_tooltip = g_main_menus[ g_main_menus.current ].tooltip
 mprint( menu_tooltip, 128,26, 9, "right" )

 if g_main_menus.submenu > 0 then
	 print( "z/🅾️:back to main menu", 40,20, 4 )
 else
	 print( "z/🅾️:end week", 1,20, 4 )
	 print( "⬆️/⬇️ then ❎", 76,20, 4 )
 end

end


------------------------------
function get_num_tenants()

 local nt = 0
 for m in all( g_guild.members ) do
  if( m.tenant ) nt += 1
 end
 
 return nt
 
end


------------------------------
function get_weekly_payroll()

 local g = g_guild
	return g.payrate * #g.members
 
end


------------------------------
function get_weekly_rent()

 local rent = 0
 for m in all( g_guild.members ) do
  if( m.tenant ) rent += g_guild.rent
 end
 
 return rent

end


------------------------------
function draw_info_bar()

end


------------------------------
function draw_menus()

 local mm = g_main_menus
 local sub_idx = mm.submenu
 for i=1,#mm do
  local x = 128-16
  local y = 16 + (16*i)
  local base_sprite = 76
  local over_sprite = i+71
  if i == mm.current then
   if sub_idx > 0 then
    pal( 7, 9 )
    pal( 6, 4 )
    pal( 1, 0 )
    pal( 5, 2 )
   else
    pal( 7, 10 )
    pal( 6, 9 )
    pal( 5, 4 )
    pal( 1, 0 )
   end
   over_sprite += 16
  else
   if sub_idx > 0 then
    pal( 7, 1 )
    pal( 6, 1 )
--    pal( 1, 0 )
    pal( 5, 1 )
   end
  end
  spr( base_sprite, x,y, 2,2 )
  pal()
  palt()
  spr( over_sprite, x+4,y+4, 1,1 )
 end

 -- line drops down and across
 if sub_idx > 0 then
  local x = 128-16
  local y1 = 16 + (16*sub_idx)
  local y2 = 96
  line( x,y1, x,y2, 9 )
  line( 0,y2, x,y2, 9 )
  
  local submenu = mm[ sub_idx ]
  submenu.draw( true )
 else
  local submenu = mm[ mm.current ]
  submenu.draw( false )
 end
 
end


------------------------------
function debug_draw_seats()

 -- draw seat #s
 for sidx = 1,#g_guild.seats do
  local seat = g_guild.seats[ sidx ]
  local x,y = seat.x + 2, seat.y + 1
  local tint = 3
  if( seat.occupant ) tint = 11
  print( sidx, x,y, tint )
 end
 
 -- draw room #s
 for ridx = 1,#g_guild.rooms do
  local room = g_guild.rooms[ ridx ]
  local x,y = room.x + 2, room.y + 1
  local tint = 13
  if( room.occupant ) tint = 12
  print( ridx, x,y, tint )
 end

end


------------------------------
function refresh_quad( quad )

 quad.cx = 16 * (2 + quad.usage)
 quad.cy = 4
 quad.sx = 0
 quad.sy = 32
 
 local idx = quad.idx
 
 if idx==3 or idx==4 then
  quad.cx += 7
  quad.sx += 56
 end
 
 if idx==2 or idx==4 then
  quad.cy += 4
  quad.sy += 32
 end

end


------------------------------
function add_new_member()

 local new = new_actor()
 add_member( new )

end


------------------------------
function add_member( member )

	add( g_guild.members, member )
	tick_actor( member )

end


------------------------------
function update_menu_remodel( in_sub )
end


------------------------------
function draw_menu_remodel( in_sub )
end


------------------------------
function update_menu_bylaws( in_sub )
end


------------------------------
function draw_menu_bylaws( in_sub )
end


------------------------------
function end_turn()

 sfx( sound_tick_tock )
 g_turn += 1
 
 local pay = get_weekly_payroll()
 local rent = get_weekly_rent()
 if rent > 0 then
  sfx( sound_get_paid )
 end
 g_guild.money += rent
 g_guild.money -= pay
 g_guild.money = mid( g_guild.money, 0, 19999 )
 
 for m in all( g_guild.members ) do
  m.money += g_guild.payrate
 end

	tick_quests()

end

----
function randomize_guildhall()

 for i=1,4 do
  local quad = g_guild.quads[i]
  quad.usage = flr(rnd(3))
 end
 
 local t = 1 + flr(rnd(4))
 g_guild.quads[t].usage = tables
 
 refresh_guild()

end



-->8
-- actors

g_next_actor_id = 1000

-- data definition tables
g_races 					= {}
g_classes 			= {}
g_genders 			= {}
g_armor_lvls = {}
g_weap_lvls 	= {}
g_wounds = { "ready", "sore", "weak", "hurt", "grave", "fatal" }

-- sprite chroma key aliases
skin  	= 15
brow  	= 9
beard 	= 5
weapon	= 14
vest			= 12
shoes		= 13

-- color aliases: armour
cloth 		= 13 -- armor=1
chain		 = 5  -- armor=2
mithril = 2  -- armor=3 (max)

-- color aliases: weapons
wood 		= 4			-- weapon=1
iron 		= 6			-- weapon=2
steel 	= 7   -- weapon=3
magic 	= 10  -- weapon=4 (max)

-------------------------------
function init_actors()

 init_races()
 init_classes()
 init_genders()
 init_weap_lvls()
 init_armor_lvls()

end


-------------------------------
function init_races()

 local human = {}
	human.name 	= "human"
	human.skin		=	15
	human.beard	=	15
	human.brow		=	15

 local dwarf = {}
 dwarf.name 	= "dwarf"
	dwarf.skin		=	15
	dwarf.beard	=	5
	dwarf.brow		=	15

 local elf = {}
 elf.name 			= "elf"
	elf.skin				= 15
	elf.beard			=	15
	elf.brow				=	0

 add( g_races, human )
 add( g_races, dwarf )
 add( g_races, elf )
 g_races.human 	= human
 g_races.dwarf		= dwarf
 g_races.elf 			= elf

 assert( g_races )
 assert( #g_races > 0 )

end


-------------------------------
function init_classes()

 local fighter = {}
 fighter.name = "fighter"
 fighter.weap = "sword"
 fighter.value = 1
 fighter.idle = { frames={20,21} }
 
 local cleric = {}
 cleric.name	 = "cleric"
 cleric.weap  = "ankh"
 cleric.value = 5
 cleric.idle	 = { frames={22,23} }

 local shaman = {}
 shaman.name = "shaman"
 shaman.weap = "ophid"
 shaman.value = 3
 shaman.idle = { frames={18,19} }

 local ranger = {}
 ranger.name = "ranger"
 ranger.weap = "bow"
 ranger.value = 2
 ranger.idle = { frames={24,25} }

 local wizard = {}
	wizard.name = "wizard"
 wizard.weap = "staff"
 wizard.value = 4
	wizard.idle = { frames={16,17} }

 add( g_classes, fighter )
 add( g_classes, cleric )
 add( g_classes, ranger )
 add( g_classes, shaman )
 add( g_classes, wizard )
 g_classes.fighter = fighter
 g_classes.cleric	 = cleric
 g_classes.ranger 	= ranger
 g_classes.shaman		= shaman
 g_classes.wizard 	= wizard

 assert( g_classes )
 assert( #g_classes > 0 )

end


-------------------------------
function init_genders()

 local male = {}
 male.name  = "male"
	male.subj 	= "he"
	male.obj			= "him"
	male.poss1 = "his"
	male.poss2 = "his"
	male.tint  = 12

 local female = {}
 female.name  = "female"
	female.subj 	= "she"
	female.obj			= "her"
	female.poss1 = "her"
	female.poss2 = "hers"
	female.tint  = 14

 local andro = {}
 andro.name  = "andro"
	andro.subj 	= "they"
	andro.obj			= "them"
	andro.poss1 = "their"
	andro.poss2 = "theirs"
	andro.tint  = 13

 add( g_genders, male )
 add( g_genders, female )
 add( g_genders, andro )
 g_genders.male 			= male
 g_genders.female 	= female
 g_genders.andro 		= andro

 assert( g_genders )
 assert( #g_genders > 0 )

end


-------------------------------
function init_weap_lvls()

 local w1 = {}
 w1.name  = "wood"
	w1.dmg   = 1
	w1.value	= 1
	w1.tint  = wood

 local w2 = {}
 w2.name  = "iron"
	w2.dmg   = 2
	w2.value	= 2
	w2.tint  = iron

 local w3 = {}
 w3.name  = "steel"
	w3.dmg   = 3
	w3.value	= 3
	w3.tint  = steel

 local w4 = {}
 w4.name  = "magic"
	w4.dmg   = 4
	w4.value	= 4
	w4.tint  = magic

 add( g_weap_lvls, w1 )
 add( g_weap_lvls, w2 )
 add( g_weap_lvls, w3 )
 add( g_weap_lvls, w4 )
 g_weap_lvls.wood	 = w1
 g_weap_lvls.iron	 = w2
 g_weap_lvls.steel	= w3
 g_weap_lvls.magic	= w4
 
 assert( g_weap_lvls )
 assert( #g_weap_lvls > 0 )

end


-------------------------------
function init_armor_lvls()

 local a1 = {}
 a1.name  = "cloth"
	a1.prot  = 1
	a1.value	= 1
	a1.tint  = cloth

 local a2 = {}
 a2.name  = "chain"
	a2.prot  = 2
	a2.value	= 2
	a2.tint  = chain

 local a3 = {}
 a3.name  = "mithril"
	a3.prot  = 3
	a3.value	= 3
	a3.tint  = mithril

 add( g_armor_lvls, a1 )
 add( g_armor_lvls, a2 )
 add( g_armor_lvls, a3 )
 g_armor_lvls.cloth	  = a1
 g_armor_lvls.chain	  = a2
 g_armor_lvls.mithril	= a3

end


-------------------------------
function new_actor()

 assert( g_races and #g_races>0 )
 assert( g_classes and #g_classes>0 )
 assert( g_genders and #g_genders>0 )

 local race_idx = 1 + flr(rnd( #g_races ))
	local class_idx = 1 + flr(rnd( #g_classes ))
	local gender_idx = 1 + flr(rnd( #g_genders ))
	
	if( rnd() < .3 ) race_idx = 1
	if( rnd() < .3 ) class_idx = 1
	if( rnd() < .5 ) gender_idx = 1 + flr(rnd(2))

 local actor = {}
 actor.name  = "jamie"
 actor.race = g_races[ race_idx ]
 actor.class = g_classes[ class_idx ]
 actor.gender = g_genders[ gender_idx ]
 
 actor.weapon = 1 + flr(min(rnd(4),rnd(4),rnd(4)))
 actor.vest 		= 1 + flr(min(rnd(3),rnd(3),rnd(3)))
 actor.shoes 	= 1 + flr(min(rnd(3),rnd(3),rnd(3)))
 actor.start  = g_turn
 actor.money  = flr(min(rnd(100),rnd(100)))

 actor.id     = g_next_actor_id
 actor.seatx		= 0
 actor.seaty 	= 0
 actor.tenant = false
 g_next_actor_id += 1
 
 assert( actor )
 assert( actor.name )
 assert( actor.race )
 assert( actor.class )
 assert( actor.gender )
 assert( actor.weapon )
 assert( actor.shoes )
 assert( actor.start )
 assert( actor.money )
 assert( actor.id )
 return actor

end


-------------------------------
function tick_actor( actor )

 -- check if their wounds heal
 -- 

end


-------------------------------
function draw_actor( actor, sx,sy, outline1, outline2 )

 assert( actor )
 assert( actor.class )
 
 local drawx = sx or actor.seatx
 local drawy = sy or actor.seaty
 
 local sprite = get_sprite( actor.class, actor.id )
 
 -- draw 16-25x +/- 2 outer outline first
 if outline2 then
  set_outline_pal( outline2 )
  for dx=-2,2 do
  for dy=-2,2 do
		 spr( sprite, drawx+dx,drawy+dy )
  end
  end
 end

 -- draw 8-9x +/- 1 inner outline next
 if outline1 then
  set_outline_pal( outline1 )
  for dx=-1,1 do
  for dy=-1,1 do
		 spr( sprite, drawx+dx,drawy+dy )
  end
  end
 end
 
 set_actor_pal( actor )
 spr( sprite, drawx,drawy )
 pal()
 palt()

end


-------------------------------
function set_actor_pal( actor )

 pal()
 palt()
 
 pal( 15, actor.race.skin )
 pal(  5, actor.race.beard )
 
 local brow = actor.race.brow
 if brow == 0 then
	 palt(  9, true )
 else
	 pal(  9, actor.race.brow )
 end

 assert( actor )
 assert( actor.weapon )
 assert( g_weap_lvls )
 assert( #g_weap_lvls > 0 ) 
 local w = g_weap_lvls[ actor.weapon ]
 local v = g_armor_lvls[ actor.vest ]
 local s = g_armor_lvls[ actor.shoes ]
 
 pal( weapon, w.tint ) 
 pal( shoes,		s.tint )
 pal( vest,   v.tint )

end


-------------------------------
function set_outline_pal( tint )

 pal()
 palt()
 
 for i=1,15 do
  pal( i, tint )
 end

end


------------------------------
function update_menu_members( in_sub )
 
-- if( not in_sub ) return

 local amenu = g_main_menus.members 
 amenu.current = left_right_wraparound( amenu.current, #g_guild.members )

end


------------------------------
function draw_menu_members( in_sub )

 local amenu = g_main_menus.members 
 local actor = g_guild.members[ amenu.current ]
 local class = actor.class
 local race = actor.race
 
 draw_actor( actor, actor.seatx, actor.seaty, 0, 7 )
 draw_actor( actor, 0,96 )

 rectfill( 8,97, 60,103, actor.gender.tint )
 print( actor.name, 9,98, 0 ) 

 local rc_text = race.name.." "..class.name
 rprint( rc_text, 128,98, 6 )
 
 local weapon = g_weap_lvls[ actor.weapon ]
 local vest  = g_armor_lvls[ actor.vest ]
 local shoes = g_armor_lvls[ actor.shoes ]
 
 local weap_name = weapon.name.." "..class.weap
 print( weap_name, 1,106, weapon.tint )

 local vest_name = vest.name.." vest"
 print( vest_name, 1,112, vest.tint ) 
 local shoes_name = shoes.name.." boots"
 print( shoes_name, 57,112, shoes.tint ) 

 local weeks = g_turn - actor.start
 local cash = actor.money
 local abode = "homeless"
 if( actor.tenant ) abode = "tenant"
 local status = "healthy"
 local desc_text = weeks.." weeks, $"..cash..", "..abode..", "..status
 print( desc_text, 1,118, 3 )
 
end



-->8
-- monsters

g_next_mob_id = 2000

-- data definition tables
g_monsters = {}

-------------------------------
function init_monsters()

 local goblin   = {}
 goblin.name 		 = "goblin"
 goblin.ac   		 = 1
 goblin.range		 = 1
 goblin.damage  = 4
 goblin.speed   = 5
 goblin.idle 			= { frames={48,49} }

 local troll   = {}
 troll.name 		 = "troll"
 troll.ac   		 = 5
 troll.regen			= 5
 troll.range		 = 1
 troll.damage  = 10
 troll.speed   = 2
 troll.weak_c 	= "wizard"
 troll.weak_r 	= nil
 troll.idle 			= { frames={56,57} }

 local zombie   = {}
 zombie.name 		 = "zombie"
 zombie.ac   		 = 1
 zombie.regen			= 2
 zombie.range		 = 1
 zombie.damage  = 8
 zombie.speed   = 1
 zombie.weak_c 	= "cleric"
 zombie.weak_r 	= nil
 zombie.idle 			= { frames={52,53} }

 local skeleton   = {}
 skeleton.name 		 = "skeleton"
 skeleton.ac   		 = 2
 skeleton.regen			= 0
 skeleton.range		 = 1
 skeleton.damage  = 6
 skeleton.speed   = 5
 skeleton.weak_c 	= "cleric"
 skeleton.weak_r 	= "dwarf"
 skeleton.idle 			= { frames={58,59} }

 add( g_monsters, goblin )
 add( g_monsters, troll )
 add( g_monsters, zombie )
 add( g_monsters, skeleton )
 g_monsters.goblin 		= goblin
 g_monsters.troll	 		= troll
 g_monsters.zombie 		= zombie
 g_monsters.skeleton = skeleton

 assert( g_monsters )
 assert( #g_monsters > 0 )

end


-------------------------------
function draw_mob( mob, sx,sy, outline1, outline2 )

 assert( mob )
 
 local drawx = sx or mob.x
 local drawy = sy or mob.y
 
 local sprite = get_sprite( mob.class, mob.id )
 
 -- draw 16-25x +/- 2 outer outline first
 if outline2 then
  set_outline_pal( outline2 )
  for dx=-2,2 do
  for dy=-2,2 do
		 spr( sprite, drawx+dx,drawy+dy )
  end
  end
 end

 -- draw 8-9x +/- 1 inner outline next
 if outline1 then
  set_outline_pal( outline1 )
  for dx=-1,1 do
  for dy=-1,1 do
		 spr( sprite, drawx+dx,drawy+dy )
  end
  end
 end
 
--set_mob_pal( mob )
	pal()
	palt()
 spr( sprite, drawx,drawy )

end


-------------------------------
function new_mob()

 assert( g_monsters and #g_monsters>0 )

 local monster_idx = 1 + min( flr(rnd(#g_monsters)), flr(rnd(#g_monsters)) )
	
 local mob = {}
 mob.name  = "dufus"
 mob.class = g_monsters[ monster_idx ]
-- mob.gender = mob
 
 mob.id     = g_next_mob_id
 mob.x		= 0
 mob.y 	= 0
 g_next_mob_id += 1
 
 assert( mob )
 assert( mob.name )
 assert( mob.class )
-- assert( mob.gender )
 assert( mob.id )
 return mob

end



-->8
-- quests

g_next_quest_id = 3000

g_quests = {}
g_max_quests = 4


------------------------------
function tick_quests()

 assert_log( g_quests )
 for quest in all(g_quests) do
  assert_log( #g_quests > 0 )
  assert_log( quest )
  if quest.completed then
   log_print( "quest #"..quest.id.." was completed.", 5 )
   del( g_quests, quest )
  else
   quest.weeks_left -= 1
   if quest.weeks_left <= 0 then
	   log_print( "quest #"..quest.id.." has expired!", 4 )
	   del( g_quests, quest )
   end
  end
 end
 
 local missing_quests = g_max_quests - #g_quests
 for i=1,missing_quests do
  if rnd() < .5 then
   local newq = new_quest()
   add( g_quests, newq )
   log_print( "added q#"..newq.id..", now "..#g_quests, 11 )
  end
 end

end


------------------------------
function update_menu_quests( in_sub )

 if( not in_sub ) return
 
 local qmenu = g_main_menus.quests
 qmenu.current = up_down_wraparound( qmenu.current, #g_quests )

end


------------------------------
function draw_menu_quests( in_sub )

 local qmenu = g_main_menus.quests

 for i=1,#g_quests do
  local q = g_quests[i]
  local x = 1
  local y = 90 + (i*8)
  local tint = 6
  if in_sub and i==qmenu.current then
   rectfill( 0,y-1, 127,y+5, 4 )
   tint = 7
  end

  -- duration remaining text  
  local weeks_text = q.weeks_left.." weeks"
  if( q.weeks_left <= 1 ) weeks_text = "ends now"
  print( weeks_text, x,y, tint )
  
  local reward_text = "+$"..q.reward
  print( reward_text, x+36,y, 11 )
  
  -- print "vs"
  spr( 119, x+56,y, 1,1 )
  
  -- print each of the (up to 3) groups
  for i=1,#q.mob_groups do
   local group = q.mob_groups[i]
	  local sx = 30 + (36 * i)
	  local sy = y
	 	local mob_type = g_monsters[ group.monster_idx ]
	 	local count = group.count
	 	rprint( tostr(count), sx+6,y, 8 )
	  spr( 120, sx+6,y, 1,1 )
--	  local mob_sprite = mob_type.idle.frames[0]
--	  spr( mob_sprite, x+69,y, 1,1 )
   print( mob_type.name, sx+9,y, 8 )
  end

 end

end


------------------------------
-- turn  0:	rewards [40,100]
-- turn 10: rewards [50,200]
-- turn 20: rewards [60,300]
-- turn 30: rewards [70,400]
function new_quest()

 local min_reward = 40 + g_turn
 local max_reward = 100 + (10 * g_turn)
 local min_groups = 1
 local max_groups = flr(sqrt(g_turn))
 local min_count = 1
 local max_count = g_turn

 local q = {}
 q.id = g_next_quest_id
 g_next_quest_id += 1
 q.weeks_left = 1 + flr(rnd(3))
 q.reward = rnd_int( min_reward, max_reward )
 local num_groups = min( rnd_int(min_groups,max_groups), rnd_int(min_groups,max_groups) )
 num_groups = mid( 1, num_groups, 3 )

 -- zero counts for each mob type
 q.mob_counts = {}
 for i=1,#g_monsters do
  add( q.mob_counts, 0 )
 end

 -- determine mob types/counts
 for i=1,num_groups do
  local m_idx = 1 + min( flr(rnd(#g_monsters)), flr(rnd(#g_monsters)) )
	 q.mob_counts[ m_idx ] += min( rnd_int(min_count,max_count), rnd_int(min_count,max_count) )
	 q.mob_counts[ m_idx ] = mid( 1, q.mob_counts[ m_idx ], 99 )
 end

 -- build monster groups from mob counts 
 q.mob_groups = {}
 for i=#q.mob_counts,1,-1 do
  if q.mob_counts[i] > 0 then
   local group = {}
   group.monster_idx = i
   group.count = q.mob_counts[i]
   add( q.mob_groups, group )
  end
 end
 
 return q

end


------------------------------
function wtf_quests()

 log_print( #g_quests.." quests:", 10 )
 assert_log( g_quests )
 for i=1,#g_quests do
  assert_log( g_quests[i] )
  log_print( "  #"..g_quests[i].id, 9 )
 end
 log_print( " ", 10 )

end


-->8
-- names and sounds


------------------------------
-- sound id aliases
sound_tick_tock 		= 1
sound_get_paid 			= 2
sound_menu_move 		= 3
sound_menu_select = 4
sound_menu_back   = 5


-->8
-- testing

g_test_pcg_actors = {}
g_test_mobs = {}

--------------------------------
function update_test_pcg()
 
 if btnp(❎) or #g_test_pcg_actors == 0 then
  g_test_pcg_actors = {}
  for i=1,144 do
   local a = new_actor()
   add( g_test_pcg_actors, a )
  end
 end
 
end


--------------------------------
function draw_test_pcg()

 cls(0)
 local slow_frame = flr( .1 * g_frame )
 local count = #g_test_pcg_actors
 local highlight1 = 1 + (slow_frame % count)
 local highlight2 = flr( 1 + ((slow_frame + count/3) % count) )
 local highlight3 = flr( 1 + ((slow_frame + count*2/3) % count) )
 for i=1,count do
  local cx = (i-1) % 12
  local cy = flr( (i-1) / 12 )
  local x = 1 + cx * 10
  local y = 1 + cy * 10
 	local actor = g_test_pcg_actors[i]

 	o1 = nil
 	o2 = nil
 	o3 = nil
 	
 	if highlight1 == i then
 	 o1 = 1 + (i % 15)
 	end
 	
 	if highlight2 == i then
 	 o1 = 0
 	 o2 = 1 + (i % 15)
 	end
 	
 	if highlight3 == i then
 	 o1 = 1 + (i*17 % 15)
 	 o2 = 1 + (i % 15)
 	end
 	
-- 	if( rnd() < .2 ) o1 = flr(rnd(16))
-- 	if( o1 and rnd() < .2 ) o2 = flr(rnd(16))
-- 	if( o2 ) o1 = 0
 	draw_actor( actor, x,y, o1, o2 )
 end

end


--------------------------------
function update_test_actors()
end


--------------------------------
function draw_test_actors()

	cls(0)

	local col_width 	= 128 / (1 + #g_races)
	local row_height = 128 / (1 + #g_classes)
	local x,y,ofs

	-- race column headers
	for race_idx = 1,#g_races do
	 local race = g_races[ race_idx ]
		x = 1 + race_idx * col_width
		y = 1
		print( race.name, x,y, 4 )
	end

 -- class row headers
	for class_idx = 1,#g_classes do
	 local class = g_classes[ class_idx ]
		x = 1
		y = 1 + class_idx * row_height
		print( class.name, x,y, 3 )
	end

 -- class/race sprite grid
 for race_idx = 1,#g_races do
  local race = g_races[ race_idx ]
		for class_idx = 1,#g_classes do
		 local class = g_classes[ class_idx ]
			x = -1 + race_idx * col_width
			y = 1 + class_idx * row_height
			local sprite1 = get_sprite( class, (x*17) + (y*27) )
			local sprite2 = get_sprite( class, (x*37) + (y*41) )
			local sprite3 = get_sprite( class, (x*29) + (y*19) )
			local sprite4 = get_sprite( class, (x*43) + (y*31) )
			local sprite5 = get_sprite( class, (x*11) + (y*13) )
			local sprite6 = get_sprite( class, (x*23) + (y*47) )
			set_clr_map( race, 1 )
			spr( sprite1, x,y-4 )
			set_clr_map( race, 2 )
			spr( sprite2, x+8,y-4 )
			set_clr_map( race, 3 )
			spr( sprite3, x+16,y-4 )
			set_clr_map( race, 4 )
			spr( sprite4, x,y+4 )
			set_clr_map( race, 5 )
			spr( sprite5, x+8,y+4 )
			set_clr_map( race, 6 )
			spr( sprite6, x+16,y+4 )
		end
	end

end


-------------------------------
function get_sprite( class, ofs )

 -- select animation sequence
	local anim = class.idle
	
	-- select anim frame -> sprite
	local nframes = #anim.frames
	local anim_fps = 8
--	local wrap_time = band( rtime(), 0x0fff )
	local raw_n_idx = ofs + flr( 0.017 * ofs +  rtime() * anim_fps )
--	raw_n_idx = flr(8 * time())
	local noise_idx = 1 + band( raw_n_idx, 0xff )
	assert( g_noise_256 != nil )
	assert( #g_noise_256 > 0 )
	assert( #g_noise_256 == 256 )
	local noise = g_noise_256[ noise_idx ]
	local frame = noise % nframes
	local sprite = anim.frames[ 1+frame ]
	return sprite

end


-------------------------------
function set_clr_map( race, n )

 pal()
 palt()
 
 pal( 15, race.skin )
 pal(  5, race.beard )
 
 local brow = race.brow
 if brow == 0 then
	 palt(  9, true )
 else
--	 palt(  9, false )
	 pal(  9, race.brow )
 end

 if n==1 then
  pal( vest, cloth )
  pal( shoes, cloth )
  pal( weapon, wood )
 elseif n == 2 then
  pal( vest, chain )
  pal( shoes, cloth )
  pal( weapon, wood )
 elseif n == 3 then
  pal( vest, cloth )
  pal( shoes, chain )
  pal( weapon, iron )
 elseif n == 4 then
  pal( vest, chain )
  pal( shoes, chain )
  pal( weapon, steel )
 elseif n == 5 then
  pal( vest, chain )
  pal( shoes, mithril )
  pal( weapon, magic )
 elseif n == 6 then
  pal( vest, mithril )
  pal( shoes, mithril )
  pal( weapon, magic )
 else
 end

end


--------------------------------
function update_test_mobs()
 
 if btnp(❎) or #g_test_mobs == 0 then
  g_test_mobs = {}
  for i=1,144 do
   local m = new_mob()
   add( g_test_mobs, m )
  end
 end
 
end


--------------------------------
function draw_test_mobs()

 cls(0)
 local slow_frame = flr( .1 * g_frame )
 local count = #g_test_mobs
 local highlight1 = 1 + (slow_frame % count)
 local highlight2 = flr( 1 + ((slow_frame + count/3) % count) )
 local highlight3 = flr( 1 + ((slow_frame + count*2/3) % count) )

 for i=1,count do
  local cx = (i-1) % 12
  local cy = flr( (i-1) / 12 )
  local x = 1 + cx * 10
  local y = 1 + cy * 10
 	local mob = g_test_mobs[i]

 	o1 = nil
 	o2 = nil
 	o3 = nil
 	
 	if highlight1 == i then
 	 o1 = 1 + (i % 15)
 	end
 	
 	if highlight2 == i then
 	 o1 = 0
 	 o2 = 1 + (i % 15)
 	end
 	
 	if highlight3 == i then
 	 o1 = 1 + (i*17 % 15)
 	 o2 = 1 + (i % 15)
 	end
 	
 	draw_mob( mob, x,y, o1, o2 )
 end

end



-->8
-- utils

g_noise_256 = {}
g_log = {}


-------------------------------
function init_utils()

 for i = 1,256 do
  add( g_noise_256, flr(rnd(256)) )
 end

end


-------------------------------
function rtime()

-- return 0.5 * time() -- for 60 fps
 return time() -- for 30 fps

end


-------------------------------
function round( number )
 return flr( number + .5 )
end


-------------------------------
function draw_stats()

 pal()
 palt()
 local mem = stat(0) / 2048
 local cpu = stat(1)
-- local fps = stat(7)

 local minx = 84
 local maxx = 94
 local wide = maxx - minx
 local memx = minx + round( mem * wide )
 local cpux = minx + round( cpu * wide )
 rectfill( minx-1,0, maxx+1,3, 1 )
 rectfill( minx,1, maxx,2, 0 )
 rectfill( minx,1, memx,1, 12 )
 rectfill( minx,2, cpux,2, 8 )

end


-------------------------------
function add_commas( number )
 
 if( number < 1000 ) return number
 
 local thousands = flr( number / 1000 )
 local ones = number - (1000 * thousands)
 if( ones < 10 ) return thousands..",00"..ones
 if( ones < 100 ) return thousands..",0"..ones
 return thousands..","..ones
 
end


-------------------------------
function mprint( msg, x,y, default_tint, align )

 local px,py = x,y

 if align then
  local predicted_length = #msg
  for i=1,#msg do
  	 local char = sub(msg,i,i)
  	 if char == "~" then
  	  predicted_length -= 2
  	 end
  end
  
  if align == "right" then
   px -= predicted_length * 4
  end
 end
 
 local skip_next = false
 local tint = default_tint
 local text = ""
 for i=1,#msg do
  if skip_next then
   skip_next = false
  else
 	 local char = sub(msg,i,i)
   if char == "~" then
    i += 1
    local tint_hex = "0x"..sub(msg,i,i)
    tint = tonum( tint_hex )
    skip_next = true
   else
 	  print( char, px,py, tint )
 	  px += 4
   end
  end
  
 end--for

end


-------------------------------
function rprint( msg, rx,y, tint )

 local pxlen = 4 * #msg
 local x = rx - pxlen
 print( msg, x,y, tint )

end


-------------------------------
function up_down_wraparound( idx, max_idx )

 local new_idx = idx
 if( btnp(⬆️) ) new_idx -= 1
 if( btnp(⬇️) ) new_idx += 1
 if( new_idx < 1 ) new_idx = max_idx
 if( new_idx > max_idx ) new_idx = 1
 if( new_idx ~= idx ) sfx( sound_menu_move )
 return new_idx
 
end


-------------------------------
function left_right_wraparound( idx, max_idx )

 local new_idx = idx
 if( btnp(⬅️) ) new_idx -= 1
 if( btnp(➡️) ) new_idx += 1
 if( new_idx < 1 ) new_idx = max_idx
 if( new_idx > max_idx ) new_idx = 1
 if( new_idx ~= idx ) sfx( sound_menu_move )
 return new_idx

end


-------------------------------
function rnd_int( min_int, max_int )

 local range = 1 + max_int - min_int
 if( range <= 0 ) return flr( 0.5 * (min_int+max_int) )
 
 return min_int + flr(rnd(range))

end


-------------------------------
function outline_print( text, x,y, text_tint, outline_tint )

 for dx=-1,1 do
 for dy=-1,1 do
  print( text, x+dx,y+dy, outline_tint )
 end
 end

 print( text, x,y, text_tint )

end


-------------------------------
function log_print( text, tint )

-- cls(0)
-- stop( "log_print: "..text )
 local new_line = {}
 new_line.text = text
 new_line.tint = tint
 add( g_log, new_line )
end


-------------------------------
function draw_log()
 
 local lines_visible = g_show_log_lines
 local first_line = 1
 if #g_log > lines_visible then
  first_line = #g_log - lines_visible + 1
 end 
 
 local x = 1
 local y = 1
 for i=first_line,#g_log do
  local l = g_log[i]
  local width = 4 * #l.text
  rectfill( x-1,y-1, x+width+1,y+5, 0 )
--  rectfill( 0,y, 127,y+5, 0 )
  print( l.text, x,y, l.tint )
--  outline_print( l.text, x,y, l.tint, 0 )
  y += 6
 end
 
end


-------------------------------
function assert_log( b )

 if( b ) return
 
 cls()
 draw_log()
 print( "" )
 assert( b )
 stop()

end


__gfx__
00000000777777760000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000700000050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00700700700aa0050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00077000700000050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00077000700aa0050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00700700700aa0050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000700aa0050000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000655555550000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00f9f00000f9f0e003f9f30003f9f30e00f9f00000f9f0e000f9f00000f9f0e00f9f0e000f9f0e00000000000000000000000000000000000000000000000000
005550e0005550400055500e00555040005550e0005550e0005550e000555e0e055500e0055500e0000000000000000000000000000000000000000000000000
000c0040000c0cf0000c0040000c0cf44e4c00e0000c0cf0000c0e0efc0c00e00050cfe4005cf4e0000000000000000000000000000000000000000000000000
0cccccf000ccc0400cccccf400ccc040eeecccf04e4ccc40fccecce00cceccf00cccc0e00cccc0e0000000000000000000000000000000000000000000000000
f00c00400c0c0040f00c00400c0c00044e4cc040eeecc0000c0c0cf0000c0c400ccc0e000ccc0e00000000000000000000000000000000000000000000000000
000c0040f00c0040000c0004f00c0040040c00004e4c0000000c0040000c000000c0000000c00000000000000000000000000000000000000000000000000000
00d0d04000d0d00000d0d04000d0d00000d0d00004d0d00000d0d00000d0d0000d0d00000d0d0000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00805000008150100181510001815101008150000081501000111000001110100111010001110100000000000000000000000000000000000000000000000000
00679010006790100067900100679010006790100067901000111010001111010111001001110010000000000000000000000000000000000000000000000000
00010010000108600001001000010161111100100001015000010101110100100010111100111110000000000000000000000000000000000000000000000000
01591860005910100159816100598010117981501119818011111110011111100111101001111010000000000000000000000000000000000000000000000000
a0070010010700101007001001070001a11a1080117a100001010110000101100111010001110100000000000000000000000000000000000000000000000000
00010010a001001000010001100100a001010000a111000000010010000100000010000000100000000000000000000000000000000000000000000000000000
0010a0100010a0000010a0a00010a000001010000110100000101000001010000101000001010000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00bbb00000bbb00000eee00000eee000009990000099900000ccc00000ccc00000ddd00000ddd000007770000077700000000000000000000000000000000000
00bbb00000bbb00000eee00000eee000009990000099900000ccc00000ccc00000ddd00000ddd000007770000077700000000000000000000000000000000000
00030000000300000002000000020000000400000004000000010000000100000005000000050000000700000007000000000000000000000000000000000000
033333b0b3333300022222e0e22222000444449094444400011111c0c1111100055555d0d5555500070707707707070000000000000000000000000000000000
b0030000000300b0e0020000000200e09004000000040090c0010000000100c0d0050000000500d0707770000077707000000000000000000000000000000000
00030000000300000002000000020000000400000004000000010000000100000005000000050000000700000007000000000000000000000000000000000000
00505000005050000050500000505000005050000050500000505000005050000050500000505000007070000070700000000000000000000000000000000000
5515551500000000000000000000000000000000000000000000000000000000000000000000000055155515000000007777777777777776aaaaaaaaaaaaaaa9
111111110000000000000000000000000000000000000000000000000000000000fff0000050500011111111000000007666666666666665a999999999999994
155515550000000000000000000000000000000000000000000000000000000000fff0605560655051551555094444907611111111111165a900000000000094
1111111100000000000000000000000000000000000000000000000000000000464500600550550011111111006f6f007611111111111165a900000000000094
5515551500000000000000000000000000000000000000000000000000000000666555f0050005005515551506f6f6007611111111111165a900000000000094
11111111000000000000000000000000000000000000000000000000000000004645504005050500111111110f6f60007611111111111165a900000000000094
5155155500000000000000000000000000000000000000000000000000000000040500000505050051551555944449007611111111111165a900000000000094
111111110000000000000000000000000000000000000000000000000000000000d0d0000005000011111111000000007611111111111165a900000000000094
5515551500000000000000000000000000000000000000000000000000000000000000004900094055155515000000007611111111111165a900000000000094
111111110000000000000000000000000000000000000000000000000000000000fff0a05a505a5011111111094444907611111111111165a900000000000094
155515550000000000000000000000000000000000000000000000000000000000fff0a0555055505155155500f7f7007611111111111165a900000000000094
1111811100008000000000000000000000000000000000000000000000000000000202f00580850011118111007f7f007611111111111165a900000000000094
55888515008880000000000000000000000000000000000000000000000000004a422240050005005588851507f7f7007611111111111165a900000000000094
1889881108898800000000000000000000000000000000000000000000000000aaa2200055050550188988110f7f70007611111111111165a900000000000094
58999888089998880000000000000000000000000000000000000000000000004a420000050505005899988807f7f0007666666666666665a999999999999994
889a9998889a99980000000000000000000000000000000000000000000000000450500000050000889a99989444490065555555555555559444444444444444
55155515551555155515551500000000000000000000000000000000000000000000000000000000000000000000000066666666666666659999999999999994
11100111111001111114411110044001000000000000000000000000000100000000000000000000000000000000000065555555555555509444444444444445
150000551500005515444455154444550000000000000000000000001000000000000000000000000e0000000000000065555555555555509400000000000045
11000011110000111144441111444411000000000000000000000000001000100000000005555550ee7777770000000065555555555555509400000000000045
55000015550002155544441555444415000000000000000000000000000000000000000055555555e77777700000000065555555555555509400000000000045
11000011110055111104441111044411000000000000000000000000000010000000000005555550444444400000000065555555555555509400000000000045
51000055510666555144445551444455000000000000000000000000001000000000000000011000505050500000000065555555555555509400000000000045
11000011117777111144441111444411000000000000000000000000000000100000000000111100500050000000000065555555555555509400000000000045
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000065555555555555509400000000000045
00010000000000000000000000000000000000000000000000000000505000000000000000000000000000000000000065555555555555509400000000000045
10000000000000000000000000000000000000000000000000000000505055005050000000000000000000000000000065555555555555509400000000000045
00100010000000000000000000000000000000000000000000000000050050000500000000000000000000000000000065555555555555509400000000000045
00000000000000000000000000000000000000000000000000000000000550005050000000000000000000000000000065555555555555509400000000000045
00001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000065555555555555509400000000000045
00100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000065555555555555509444444444444445
00000010000000000000000000000000000000000000000000000000000000000000000000000000000000000000000050000000000000004555555555555555
__label__
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000fff0000000fff0000000f0f0400000fff0000000fff060000fff04000000fff0600000fff0400000fff0000003fff3000000fff000000fff0a00000000000
0005550400000fff0600000fff0400000fff040000055506000055500400000fff0600000fff0400000fff0700000fff0060000fff040000fff00a0000000000
00005040400000d004000000505f0004445004000000d0df00000502f4400000d0df000000505f000000d004000000d004000444200400000fdf4a0000000000
0f5545540000dddddf0004445554000444555f000464ddd40000222204000464ddd400000555040000dddddf0000dddddf400444222f0000dddd0a0000000000
0050505f000f00d00400044455000004445504000666dd000000222040000666dd000000505004000f00d004000f00d00400044422040000ddd0a00000000000
00005004000000d00400044450000000405000000464d0000000020000000464d000000f005004000000d004000000d000400040200000000d00000000000000
000d0d0000000d0d040000420200000005050000004d0d00000020200000004d0d0000000505000000050504000005050400000505000000d0d0000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
003fff3000000fff0000000fff0400003fff3040000fff0400003fff3040000f0f0000000fff0400000fff0600000f0f0000000fff000000f0f0400000000000
00055500a0000fff06000005554040000fff0400000fff04000005550400000fff0600000fff0400000fff0600000fff0700000fff060000fff0040000000000
000050040004645006000f505004000000505f400000202f000000d0df4004645006000000505f000000d0df0000005004000000500400000fdf440000000000
0055555f400666555f000055455f0000055504000444222400000ddd04000666555f0004445554000464ddd4000055555f000055555f0000dddd040000000000
0f00500400046455040000005054000050500040044422000000d0d00040046455040004445500000666dd00000f005004000f0050040000ddd0400000000000
0000500040004050000000005000000f0050040004442000000f00d00400004050000004445000000464d0000000005004000000500400000d00000000000000
000d0d0400000d0d000000050500000002020000004d0d0000000505000000050500000045050000004d0d00000005050400000505040000d0d0000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00fff04000000fff060000fff06000003fff3000000fff0000000fff0400000f0f0a00000fff060000fff04000003fff3040000fff0700000fff000000000000
00fff00400000fff06000055500600000fff0070000fff04000005554040000fff0a00000555040000fff00400000fff0400000fff0700000555040000000000
000fdf44000000d0df000005df46000000d004000444d004000f505004000000505f000000202f00000f05f4400000d0df400000202f00000050040000000000
00dddd04000464ddd40000dddd060000dddddf400444dddf000055455f0004a455540000022204000055550400000ddd040004742224000055555f0000000000
00ddd040000666dd000000ddd060000f00d004000444dd040000005054000aaa5500000020200400005550400000d0d0004007772200000f0050040000000000
000d0000000464d00000000d0000000000d000400040d00000000050000004a45000000f0020040000050000000f00d004000474200000000050040000000000
0050500000004202000000d0d00000000d0d0400000505000000020200000045050000000d0d000000d0d000000005050000004d0d0000000d0d040000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000fff000000fff06000000fff0400000fff0700000fff0400000fff0000000fff0000000fff0000000fff0000000fff0400000fff0700000fff060000000000
000fff070000555006000005550400000fff0700000fff0400000fff0400000fff0700000fff0600000fff0700000fff0400000fff7070000fff040000000000
0000d07070000505f6400000505f000000505f000000505f000444d0040004745007000464d006000000d070700000d0df000f505007000000d0df0000000000
0fdd7dd70000555506000444555400047455540004445554000444dddf000777555f000666dddf000fdd7dd7000444ddd4000055755f00000ddd040000000000
00d0d0df0000555060000444550000077755000004445500000444dd040004745504000464dd040000d0d0df000444dd0000000050540000d0d0040000000000
0000d0040000050000000444500000047450000004445000000040d0000000405000000040d000000000d004000444d0000000005000000f00d0040000000000
000505000000d0d00000004d0d0000004d0d0000004d0d0000000d0d0000000d0d0000000d0d0000000505000000450500000005050000000505000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
003fff3060000fff0000000fff0700003f0f30a0000f0f0000000fff0000000fff0400000fff0a00000fff0000000f0f0000003fff3000003f0f304000000000
0005550400000fff07000005550700000fff0400000fff04000005550600000fff04000005550a00000fff0400000fff04000005550060000fff040000000000
0000d0df400000d004000000505f000000505f4004445004000464d006000000d0df000000d0df0004445004000444d004000000d004000000505f4000000000
000ddd040000dddddf00047455540000055504000444555f000666dddf00000ddd040004a4ddd4000444555f000444dddf0000dddddf40000555040000000000
00d0d000400f00d004000777550000005050004004445504000464dd040000d0d004000aaadd000004445504000444dd04000f00d00400005050004000000000
0f00d004000000d0040004745000000f0050040000405000000040d000000f00d0040004a4d0000000405000000040d000000000d000400f0050040000000000
000505000000050504000045050000000d0d0000000d0d00000005050000000d0d00000045050000000505000000050500000005050400000d0d000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000fff0400000f0f0000000fff0000000fff070000fff04000000fff0400003f0f3000000f0f0600000f0f0000000fff0000000fff0000003fff304000000000
000fff4040000fff06000005550700000fff707000fff004000005550400000fff0040000fff0600000fff0600000fff0400000fff0600000fff040000000000
0fd0d004000000d004000474d007000fd0d00700000f02f4400000d0df000000d004000000d0df000464d006000000d004000464d006000000505f4000000000
00dd4ddf0000dddddf000777dddf0000dd7ddf0000222204000444ddd40000dddddf400464ddd4000666dddf0000dddddf000666dddf00000555040000000000
0000d0d4000f00d004000474dd04000000d0d40000222040000444dd00000f00d004000666dd00000464dd04000f00d004000464dd0400005050004000000000
0000d000000000d004000040d000000000d0000000020000000444d000000000d000400464d000000040d000000000d004000040d000000f0050040000000000
000202000000050504000005050000000202000000d0d00000004d0d0000000505040000450500000005050000000d0d04000005050000000505000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00f0f04000000fff0600003fff300000fff06000000fff0400003fff3000000f0f0600000fff000000fff06000000f0f0400000fff0400003fff300000000000
00fff00400000fff0600000fff00a000fff00600000fff0400000fff0060000fff0600000fff04000055500600000fff0400000fff4040000fff004000000000
000f0df4400000505f000000500400000f0df6400000d0df0000005004000000d0df000444d0040000050df6400000d0df000f20200400000050040000000000
00dddd040004645554000055555f4000dddd06000444ddd4000055555f400464ddd4000444dddf0000dddd06000444ddd4000022422f000055555f4000000000
00ddd0400006665500000f0050040000ddd060000444dd00000f005004000666dd00000444dd040000ddd060000444dd000000002024000f0050040000000000
000d00000004645000000000500040000d0000000444d0000000005000400464d000000040d00000000d0000000444d000000000200000000050004000000000
00d0d00000004d0d0000000d0d04000050500000004d0d0000000d0d0400004d0d0000000202000000d0d0000000450500000005050000000505040000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
003fff3000000fff0a00000fff000000fff0a00000fff06000000fff000000f0f04000000fff0600000fff000000fff07000000fff0400000fff040000000000
0005550060000fff0400000555070000fff00a0000fff00600000fff040000fff00400000fff6060000fff040000fff00700000fff0400000fff040000000000
00005004000000d0df000474200700000fdf4a00000fdf46000444500400000f0df4400f505006000000d04040000fdf47000000505f000000505f0000000000
0055555f40000ddd04000777222f0000dddd0a0000dddd06000444555f0000dddd04000055655f000fdd4dd40000dddd07000005550400044455540000000000
0f0050040000d0d00400047422040000ddd0a00000ddd06000044455040000ddd04000000050540000d0d0df0000ddd070000050500400044455000000000000
00005000400f00d004000040200000000d000000000d0000000040500000000d00000000005000000000d00400000d0000000f00500400044450000000000000
0005050400000d0d0000000d0d000000202000000050500000000d0d00000020200000000d0d0000000d0d000000505000000005050000004d0d000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000fff0000000fff0000000fff0700000fff0000000fff0000003fff3000003fff3040000fff0a00000fff0000000fff040000fff04000000fff000000000000
000fff0600000fff0600000fff0400000fff0700000fff0400000fff0060000fff0400000fffa0a0000fff0a00000fff040000fff00400000555040000000000
0000d004000464d006000000d0df000000d0707000005040400000d004000000202f400f50500a0004a4d00a000000d0df00000f2f44000444d0040000000000
00dddddf000666dddf00000ddd04000fdd7dd7000f5545540000dddddf4000022204000055a55f000aaadddf00000ddd040000222204000444dddf0000000000
0f00d004000464dd040000d0d0040000d0d0df000050505f000f00d004000020200040000050540004a4dd040000d0d0040000222040000444dd040000000000
0000d004000040d000000f00d004000000d0040000005004000000d000400f0020040000005000000040d000000f00d0040000020000000040d0000000000000
0002020400000d0d0000000d0d0000000d0d00000005050000000d0d0400000202000000050500000005050000000d0d00000050500000000505000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000fff0000000fff0000000fff0700000fff0400003fff3040000fff0000000fff0600000fff0000000fff0600000fff000000fff04000000f0f060000000000
0005550700000fff0700000fff0700000fff0400000fff0400000fff0600000fff0400000fff0400000fff0400000555040000fff00400000fff060000000000
000050040004745007000000d0df000000505f000000d0df400464d006000000505f000444d004000000505f000444500400000f05f4400000202f0000000000
0055555f000777555f000474ddd4000444555400000ddd04000666dddf0000055504000444dddf0000055504000444555f000055550400046422240000000000
0f0050040004745504000777dd0000044455000000d0d000400464dd040000505004000444dd0400005050040004445504000055504000066622000000000000
000050040000405000000474d0000004445000000f00d004000040d000000f005004000040d000000f0050040000405000000005000000046420000000000000
000d0d0400000d0d0000004202000000450500000005050000000d0d00000005050000000d0d0000000d0d0000000d0d00000020200000004202000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000fff0000003fff3000003fff3040000fff0000000f0f070000fff06000000f0f0400000fff0000000f0f0700000fff0000000fff0400000f0f070000000000
00055506000005550040000fff0400000fff0700000fff04000055500600000fff04000005550400000fff0700000fff0600000fff0400000fff040000000000
0000d004000000d004000000d0df400474d007000000505f00000505f6400000d0df000444d004000000505f000464d006000000d0df000000202f0000000000
00dddddf0000dddddf40000ddd04000777dddf00000555040000555506000444ddd4000444dddf0004745554000666dddf00000ddd0400000222040000000000
0f00d004000f00d0040000d0d000400474dd0400005050040000555060000444dd00000444dd040007775500000464dd040000d0d00400002020040000000000
0000d004000000d000400f00d004000040d000000f0050040000050000000444d000000040d0000004745000000040d000000f00d004000f0020040000000000
0005050400000d0d0400000d0d0000000d0d0000000d0d00000050500000004d0d00000005050000004d0d0000000d0d0000000d0d0000000d0d000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
003fff300000fff04000000fff060000fff04000000f0f0600000fff0000000fff0400000fff040000f0f04000000f0f0000000f0f0600000fff000000000000
000fff004000fff00400000fff606000fff00400000fff0600000fff0600000fff0400000fff040000fff00400000fff0700000fff6060000fff060000000000
0000d00400000f02f4400f50500600000f5f44000000505f000000d004000000d0df000000202f00000f05f4400000d004000fd0d00600000050040000000000
00dddddf4000222204000055655f000055550400046455540000dddddf000444ddd4000444222400005555040000dddddf0000dd6ddf000055555f0000000000
0f00d0040000222040000000505400005550400006665500000f00d004000444dd0000044422000000555040000f00d004000000d0d4000f0050040000000000
0000d0004000020000000000500000000500000004645000000000d004000444d00000044420000000050000000000d004000000d00000000050040000000000
000d0d040000d0d0000000020200000050500000004505000000020204000042020000004d0d00000050500000000d0d04000005050000000505040000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000

__map__
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
4061405050404040405050404040010100000000000000000000000000000101404040404040404040404040404001014061405050404040405050404040010140624062406240406240624062400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
4070707070707070707070707040010100000000000000000000000000000101404040404040404040404040404001014070677070677070677070677040010140704070407040407040704070400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
40706970706970706970706970400101000000000000000000000000000001014040404040404040404040404040010140676967676967676967676967400101406a406a406a40406a406a406a400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
4070707070707070707070707040010100000000000000000000000000000101404040404040404040404040404001014070677070677070677070677040010140404040404040404040404040400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
4070707070707070707070707040010100000000000000000000000000000101404040404040404040404040404001014070677070677070677070677040010140404040404040404040404040400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
4070697070697070697070697040010100000000000000000000000000000101404040404040404040404040404001014067696767696767696767696740010140704070407040407040704070400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
40707070707070707070707070400101000000000000000000000000000001014040404040404040404040404040010140706770706770706770706770400101406a406a406a40406a406a406a400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
4040404040404040404040404040010100000000000000000000000000000101404040404040404040404040404001014040404040404040404040406340010140634063406340406340634063400101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
__sfx__
000200000c0100c0100e0100e0101b01019010170101501013010100100e01009010050100101001010050100a0100f01013010160101601016010140101001008010010100201006010090100a0100801002010
0002000033610336003360033600000000000000000000000000000000106101060010600106001a4001c4001d4001d4001f40033610336003360033600000000000000000000000000010610106001060010600
000200003f0703f0703f0603f0403f0303f0103f0003f0003f0703f0703f0603f0503f0303f0103f0003f0003f0703f0703f0603f0503f0303f0103f0003f0703f0603f0403f0203f0103f0003f0003f0003f000
000200001202000030010000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
000300001e0100e0700e0400101001010010100101001010010100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00040000120101e020010000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
__music__
00 01424344
00 01024344

